/*
 * @(#)Clause.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge;

/**
 * The clause category for a FD. This class is used whenever users
 * want to create a clause. As the minimum requirement, this class
 * must be used together with the Partic and Proc class. In addition
 * to setting the process and the participant of a clause, there are
 * methods in this class to set the various properties of a
 * clause. For methods that set the properties of a clause such as the
 * setTense(...) and setPolarity(...) methods, some of the possible
 * values/options for these methods are provided in the FDProperties
 * class.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ClauseFactory
 * @see Proc
 * @see Partic
 * @see FDProperties
 */

public interface Clause extends FD {
    /**
     * A method to set the process of a clause. (Required for a clause)
     * @param proc the process to be specified
     */
    public void setProc(Proc process);

    /**
     * A method to set the participants of a clause. (Required)
     * @param partic the participants to be specified
     */
    public void setPartic(Partic partic);

    /**
     * A method to allow the user to set the tense of the clause. (Optional)
     * @param tense the tense of the clause
     */
    public void setTense(String tense);

    /**
     * A method to set the adverb of a clause. (Optional)
     * @param adverb the adverb to be inserted
     */
    public void setAdverb(String adverb) throws InvalidPathException;

    /**
     * A method to set the restrictiveness of a clause. (Optional)
     * @param restrictiveness true to be restrictive, false to be non-restrictive
     */
    public void setRestrictive(boolean restrictiveness);

    /**
     * A method to set the polarity of a clause. By default, the polarity
     * is positive. (Optional)
     * @param polarity the polarity of a clause
     */
    public void setPolarity(String polarity);

    /**
     * A method to set the mood of a clause. (Optional)
     * @param mood the mood of the clause
     */
    public void setMood(String mood);

    /**
     * A method to set the scope of a clause by specifying the path of
     * the holes in an array. (Optional)
     * @param scope specify the order of the roles by putting them in
     * the right order into the array. For example, (scope {^ A B})
     * will be {"A", "B"}.
     */
    public void setScope(String[] scope);

    /**
     * A method to set the scope of a clause by making a referrence to
     * the desired hole of the clause. If the hole is empty without
     * any properties, it will be a FDNull object. Otherwise, the hole
     * will just be a normal FD but it will be referrenced to the
     * clause as the hole.
     * @param ref the FD node which serves as the hole of the clause.
     * @see FDNull
     */
    public void setScopeUsingRef(FDRef ref);

    /**
     * A method to set the complexity of a clause. (Optional)
     * @param complex the complexity to be specified
     */
    public void setComplex(String complex);

    /**
     * A method to set the circumstance of a clause. (Optional)
     * @param circum the circumstance to be specified
     */
    public void setCircum(Circum circum);

    /**
     * A method to set the pre-modifier of a clause. (Optional)
     * @param preMod the pre-modifier to be specified
     */
    public void setPreMod(PreMod preMod);
}
