/*
 * @(#)FD.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */


package edu.columbia.surge;
import java.util.Map;

/**
 * The interface which consists of the primary functions and features
 * of a FD. A lot of the functions are derived from the java.util.Map
 * interface. This is the base interface which is inherrited by
 * various other kinds of FDs such as Proc, Clause, Partic... As this
 * class defines what all kinds of FDs can do, it provides
 * comprehensive methods which allow users to manipulate the path and
 * the value of a FD. Besides being able to create new FDs by
 * supplying its name and its value, there are also methods that allow
 * users to supply a given path and a value so that users can traverse
 * through the FDs with the given path names and change the value that
 * is at the end of the path.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see FDFactory
 * @see AP
 * @see Circum
 * @see Clause
 * @see Common
 * @see FDNull
 * @see FDRef
 * @see NP
 * @see Partic
 * @see Proc
 * @see PersonalPronoun
 * @see PP
 * @see PreMod
 * @see Proper
 */

public interface FD extends Map {
    /**
     * A method that outputs the FD to an XML.
     *
     * @return some kind of an XML object that represents the FD
     * @exception InvalidPathException
     */
    public Object toXML() throws InvalidPathException;

    /**
     * A method for user to put a new value object into the key of a
     * FD that is specified by the given existing path. If the path is
     * not found, an exception will be thrown and no object will be
     * inserted.
     *
     * @param path A path name which users specify the path to the
     *   desired node with a String array which each elelemnt in the path
     *   array will be the name of the node to be traversed.
     * @param newValue A new value to be placed in the node of the given path
     * @exception InvalidPathException
     */
    public void putPath(String[] path, Object newValue) throws InvalidPathException;

    /**
     * A method for user to put a new value object into the key of a
     * FD that is specified by the given existing path. If the path is
     * not found, an exception will be thrown and no object will be
     * inserted.
     *
     * @param path A path name which users specify the path to the
     *   desired node. The path is specified in one string with each
     *   path separated by spaces.
     * @param newValue A new value to be placed in the node of the
     *   given path
     * @exception InvalidPathException
     */
    public void putPath(String path, Object newValue) throws InvalidPathException;

    /**
     * A method for user to put a new value object associate with the
     * key of a FD that is specified by the given existing path. If
     * the given path is not found, it will be automatically created
     * and the new object will be inserted at the end of the path.
     *
     * @param path A path name which users specify the path to the
     *   desired node with a String array which * each elelemnt in the
     *   path array will be the name of the node to be traversed.
     * @param newValue A new value to be placed in the node of the
     *   given path
     * @exception InvalidPathException
     */
    public void putPathOverride(String[] path, Object newValue) throws InvalidPathException;

    /**
     * A method for user to put a new value object associate with the
     * key of a FD that is specified by the given existing path. If
     * the given path is not found, it will be automatically created
     * and the new object will be inserted at the end of the path.
     *
     * @param path A path name which users specify the path to the
     *   desired node with a string which * path names are separated by
     *   spaces.
     * @param newValue A new value to be placed in the node of the
     *   given path
     * @exception InvalidPathException
     */
    public void putPathOverride(String path, Object newValue) throws InvalidPathException;

    /**
     * A method to verify whether the path exists in this FD.
     *
     * @param path the path to be verified, specified by a string
     *   array with the names of the nodes of the path.
     * @return a boolean of the verification result
     */
    public boolean checkPath(String[] path);

    /**
     * A method to verify whether the given path exists in this FD.
     *
     * @param path the path to be verified, specified by a string with
     *   the names of the nodes of the path separated by spaces.
     * @return a boolean of the verification result
     */
    public boolean checkPath(String path);

    /**
     * A method to return the object that associates with the given path.
     *
     * @param path the path of the object to be retrieved, specified
     *   in a string array that stores each path node name.
     * @return value the object on that path.
     * @exception InvalidPathException
     */
    public Object getPath(String[] path) throws InvalidPathException;

    /**
     * A method to return the object that associates with the given path.
     *
     * @param path the path of the object to be retrieved, specified
     *   in a string which each path name is separated by one space.
     * @return value the object on that path.
     * @exception InvalidPathException
     */
    public Object getPath(String path) throws InvalidPathException;

     /**
      * Override the toString() method inherrited from HashMap, this
      * method unifies the FD into its equivalent lisp-string
      * representation.
      *
      * @return the lisp-string representation of the FD
      */
    public String toString();
}
