/*
 * @(#)Partic.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge;

/**
 * The participant of a FD in a clause. This is used when user creates
 * a clause and thus is used together with the Proc and Clause
 * class. Depends on what the process is, users will only need to use
 * the ones that they need for that particular process.  However, here
 * I try to provide all kinds of information for users to composite
 * various kinds of participant. There are methods to set the subject
 * and object of the participant independently, and there are also
 * methods to set up particular subject/object pairs since for the
 * most cases, there usually has a pair of elements in a participant
 * of a clause. By using the "pair-methods" to setup participant also
 * help users to reduce the risk of grouping incompatible
 * subject/object pair. In the "pair-methods", I try to summarize in
 * what scenarios will user need to use that particular
 * method. However, the information that i gave there is not
 * complete. User should always follow the spec of SURGE since there
 * may be other different combinations. Moreover, in some cases, there
 * will be two pair of participants in each scenario. In those cases,
 * users can create a pair using the constructor and create another
 * pair using one of the set methods in this class. User can either
 * use the ParticFactory class or directly use the constructor of this
 * class to create instances of Partic.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ParticFactory
 * @see Clause
 * @see Proc */

public interface Partic extends FD {
    /**
     * A method to set the agent of the participant
     * @param agent the agent of the participant
     */
    public void setAgent(FD agent);

    /**
     * A method to set the range of the participant
     * @param range the range of the participant
     */
    public void setRange(FD range);

    /**
     * A method to set the created of the participant
     * @param created the created of the participant
     */
    public void setCreated(FD created);

    /**
     * A method to set the affected of the participant
     * @param affected the affected of the participant
     */
    public void setAffected(FD affected);

    /**
     * A method to set the processor of the participant
     * @param processor the processor of the participant
     */
    public void setProcessor(FD processor);

    /**
     * A method to set the phenomenon of the participant
     * @param phenomenon the phenomenon of the participant
     */
    public void setPhenomenon(FD phenomenon);

    /**
     * A method to set the sayer of the participant
     * @param sayer the sayer of the participant
     */
    public void setSayer(FD sayer);

    /**
     * A method to set the addressee of the participant
     * @param addressee the addressee of the participant
     */
    public void setAddressee(FD addressee);

    /**
     * A method to set the verbalization of the participant
     * @param verbalization the verbalization of the participant
     */
    public void setVerbalization(FD verbalization);

    /**
     * A method to set the carrier of the participant
     * @param carrier the carrier of the participant
     */
    public void setCarrier(FD carrier);

    /**
     * A method to set the attribute of the participant
     * @param attribute the attribute of the participant
     */
    public void setAttribute(FD attribute);

    /**
     * A method to set the identified of the participant
     * @param identified the identified of the participant
     */
    public void setIdentified(FD identified);

    /**
     * A method to set the identifier of the partic
     * @param identifier the identifier of the partic
     */
    public void setIdentifier(FD identifier);

    /**
     * A method to set the possessor of the partic
     * @param possessor the possessor of the partic
     */
    public void setPossessor(FD possessor);

    /**
     * A method to set the possessed of the partic
     * @param possessed the possessed of the partic
     */
    public void setPossessed(FD possessed);

    /**
     * A method to set the located of the partic
     * @param located the located of the partic
     */
    public void setLocated(FD located);

    /**
     * A method to set the accompaniment
     * @param accompaniment the accompaniment
     */
    public void setAccompaniment(FD accompaniment);

    /**
     * A method to set the time
     * @param time the time
     */
    public void setTime(FD time);

    /**
     * A method to set the location
     * @param location the location
     */
    public void setLocation(FD location);

    /**
     * A method to set the agent-range pair; this method is used
     * when we have the effect-type of with-range.
     * @param agent the agent
     * @param range the range
     */
    public void setAgentRangePair(FD agent, FD range);

    /**
     * A method to set the agent-location pair; this method is used.
     * @param agent the agent
     * @param location the location
     */
    public void setAgentLocationPair(FD agent, FD location);

    /**
     * A method to set the agent-created pair; this is used when we
     * have the creative effect-type.
     * @param agent the agent
     * @param created the created
     */
    public void setAgentCreatedPair(FD agent, FD created);

    /**
     * A method to set the agent-affected pair; this is used when we
     * have the dispositive effect-type.
     * @param agent the agent
     * @param affected the affected
     */
    public void setAgentAffectedPair(FD agent, FD affected);

    /**
     * A method to set the processor-phenomenon pair; this is used
     * when we have the mental process type.
     * @param processor the processor
     * @param phenomenon the phenomenon
     */
    public void setProcessorPhenomenonPair(FD processor, FD phenomenon);

    /**
     * A method to set the sayer-addressee pair; this is used when we
     * have the verbal process type.
     * @param sayer the sayer
     * @param addressee the addressee
     */
    public void setSayerAddresseePair(FD sayer, FD addressee);

    /**
     * A method to set the sayer-verbalization pair; this is used when
     * we have the verbal process type.
     * @param sayer the sayer
     * @param verbalization the verbalization
     */
    public void setSayerVerbalizationPair(FD sayer, FD verbalization);

    /**
     * A method to set the carrier-attribute pair; this is used when
     * we have the ascriptive process type and attributive
     * effect-type.
     * @param carrier the carrier
     * @param attribute the attribute
     */
    public void setCarrierAttributePair(FD carrier, FD attribute);

    /**
     * A method to set the identified-identifier pair; this is used
     * when we have the ascriptive process type and the equative
     * effect-type.
     * @param identified the identified
     * @param identifier the identifier
     */
    public void setIdentifiedIdentifierPair(FD identified, FD identifier);

    /**
     * A method to set the possessor-possessed pair; this is used when we have
     * the possessive process type and attributive effect-type.
     * @param possessor the possessor
     * @param possessed the possessed
     */
    public void setPossessorPossessedPair(FD possessor, FD possessed);

    /**
     * A method to set the located-accompaniment pair; this is used
     * when you have the locative and accompaniment type.
     * @param located
     * @param accompaniment
     */
     public void setLocatedAccompanimentPair(FD located, FD accompaniment);

     /**
      * A method to set the located-time pair; this is used when you
      * have a locative and temporal type.
      * @param located
      * @param time
      */
     public void setLocatedTimePair(FD located, FD time);

     /**
      * A method to set the located-location pair; this is used when
      * you have a spatial and attributive type.
      * @param located
      * @param location
      */
     public void setLocatedLocationPair(FD located, FD location);
}
