/*
 * @(#)CircumH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * The circumstance category. This is often used together with the
 * Clause, Proc and Partic classes because the parimary usage of a
 * circumstance is to describe the settings of the process of a
 * clause. This class has methods to set the lexical entry which tells
 * the circumstance and also has methods to set the characteristics of
 * the circumstance.  This is the class that implements the Circum
 * interface and is implemented in the form of a HashMap as it is
 * inherrited from the FDH class. This class consists of methods and
 * constructors to set the required components of a CircumH. User can
 * either use the CircumFactoryH class or directly use the constructor
 * of this class to create instances of CircumH.  purpose.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see Clause
 * @see Proc
 * @see Partic
 * @see PreMod
 * @see CircumFactoryH
 */

public class CircumH extends FDH implements Circum {
    /**
     * Default Constructor which creates a default instance of CircumH
     * object without any parameters.
     * @return a default instance of CircumH.
     */
    public CircumH() {
	super();
    }

    /**
     * Constructor which creates an instance of CircumH object with
     * the given entries that tell and specify the circumstance. If
     * invalid parameters are input, an exception will be thrown.
     * @param propertyName the name of the circumstance property
     * @param propertyBody the body of the circumstance property
     * @param lex the possible lexical entry of the circumstance
     * @return an instance of CircumH instantiated with the property
     * name and the specified properties.
     * @exception InvalidConstructorException
     */
    public CircumH(String propertyName, FD propertyBody, String lex) 
	throws InvalidConstructorException {
	super();
	if( propertyName == null || propertyName.equals("") )
	    throw new InvalidConstructorException("No property name.");
	if( propertyBody == null )
	    throw new InvalidConstructorException("No property body.");
	this.put(propertyName, propertyBody);
	if( lex != null && !lex.equals("") )
	    this.setLex(lex);
    }

    /**
     * The factory method which creates an instance of CircumH object
     * with the given entries that tell and specify the
     * circumstance. This constructor accounts for all possible
     * cirsumtances that users can specify. Among all the possible
     * input circumstance characteristics, users should only enter one
     * characteristic and set the rest of them be null. Otherwise, if
     * more than one characteristics is inputted, an exception should
     * be thrown. If invalid parameters are input, an exception will
     * be thrown.
     * @param behalf the behalf characteristics of the circumstance
     * @param effect the effect characteristics of the circumstance
     * @param inLoc the in-location characteristics of the
     * circumstance
     * @param manner the manner characteristics of the circumstance
     * @param purpose the purpose characteristics of the circumstance
     * @param reason the reason characteristics of the circumstance
     * @param time the time characteristics of the circumstance
     * @param lex the possible lexical entry of the circumstance
     * @return an instance of CircumH instantiated with the property
     * name and the specified properties.
     * @exception InvalidConstructorException
     */
    public CircumH(FD behalf, FD effect, FD inLoc, FD manner, FD purpose, 
		   FD reason, FD time, String lex) 
	throws InvalidConstructorException {
	super();
	if(behalf != null)
	    this.setBehalf(behalf);
	if(effect != null)
	    this.setEffect(effect);
	if(inLoc != null)
	    this.setInLoc(inLoc);
	if(manner != null)
	    this.setManner(manner);
	if(purpose != null)
	    this.setPurpose(purpose);
	if(reason != null)
	    this.setReason(reason);
	if(time != null)
	    this.setTime(time);
	if( lex != null && !lex.equals("") )
	    this.setLex(lex);
    }

    /**
     * A method to set the lexical entry of the circumstance.
     * @param lex
     */
    public void setLex(String lex) {
	this.put("lex", lex);
    }

    /**
     * A method to set the behalf of the circumstance.
     * @param behalf
     */
    public void setBehalf(FD behalf) {
	this.put("behalf", behalf);
    }

    /**
     * A method to set the effect of the circumstance.
     * @param effect
     */
    public void setEffect(FD effect) {
	this.put("effect", effect);
    }

    /**
     * A method to set the in-loc of the circumstance.
     * @param in_loc
     */
    public void setInLoc(FD in_loc) {
	this.put("in-loc", in_loc);
    }

    /**
     * A method to set the manner of the circumstance.
     * @param manner
     */
    public void setManner(FD manner) {
	this.put("manner", manner);
    }

    /**
     * A method to set the purpose of the circumstance.
     * @param purpose
     */
    public void setPurpose(FD purpose) {
	this.put("purpose", purpose);
    }

    /**
     * A method to set the reason of the circumstance.
     * @param reason
     */
    public void setReason(FD reason) {
	this.put("reason", reason);
    }

    /**
     * A method to set the time of the circumstance.
     * @param time
     */
    public void setTime(FD time) {
	this.put("time", time);
    }
}
