/*
 * @(#)ClauseFactoryH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * This class holds the factory methods for the Clause class. Users
 * can use this class to create instances of Clause objects with their
 * desired parameters. A static keyword is used to instantiate an
 * instance of ClauseFactoryH so that the static variable "factory" in
 * this class will have access to the create methods. Users can also
 * create instances of ClauseH directly using the constructors of the
 * Clause class. The primary reason of the creation of this class is
 * to allow the Clause class, which implements the Clause interface to
 * have constructors while being able to implement all the methods in
 * the interface.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ClauseH
 */

public class ClauseFactoryH implements ClauseFactory {
    /**
     * The static instance which helps users to access to the methods
     * in this class.
     */
    public static ClauseFactoryH factory;

    /**
     * A private constructor so that instances of this class cannot be
     * instantiated except using the static variable- "factory".
     */
    private ClauseFactoryH() { }

    /**
     * Invoke an instance of the ClauseFactoryH so that the factory
     * variable will have access to the methods of this class.
     */
    static {
	factory = new ClauseFactoryH();
    }

    /**
     * A factory method which creates an instance of ClauseH without
     * any given parameters.
     */
    public Clause create() {
	return new ClauseH();
    }

    /**
     * A factory method to compute and return an instance of Clause
     * class that fulfills the minimum requirement of a clause, which
     * consists of a process and a participant. An
     * InvalidConstructorException is thrown if the proc and partic
     * parameters are invalid. And an InvalidPathException will be
     * thrown when the program is failed to use the putPathOverride()
     * method in the FDH class to construct the clause FD.
     * @see FDH#putPathOverride
     * @param proc the process of the clause
     * @param partic the participant of the cluase
     * @return an instance of Clause which consists of a given process
     * and a participant
     * @exception InvalidConstructorException
     * @exception InvalidPathException
     */
    public Clause create(Proc proc, Partic partic) 
	throws InvalidConstructorException, InvalidPathException {
	return new ClauseH(proc, partic);
    }

    /**
     * A factory method to compute and return an instance of ClauseH
     * class that gives user the option to create an instance of
     * ClauseH with all the most common and frequently used
     * properties. Among all the parameters, only proc and partic are
     * required. If users do not want to specify certain properties,
     * simply put a null as the actual parameter. For some parameters
     * such as the "tense", "mood", the possible values/options can be
     * found in the FDProperties class.
     * @param proc the process of the clause
     * @param partic the participant of the clause
     * @param tense the tense of the clause
     * @param adverb the possible adverb for the clause
     * @param mood the possible mood of the clause
     * @param polarity the polarity of the clause
     * @param complexity set whether the clause is complex, true to be
     * complex and vice versa.
     * @param restrictiveness set whether the clause is restrictive,
     * true to be restrictive and vice versa.
     * @param circum the circumstance of the clause
     * @param preMod the predefined-modifier of the clause
     * @param scope specify the path to the hole and put them in the
     * right order into the array. For example, if the whole is
     * (partic (agent..))  then put String[] scope = {"partic",
     * "agent"}
     * @param ref the FD node which serves as the hole of the clause.
     * @return an instance of Clause which consists of all the user
     * specifications.
     * @exception InvalidConstructorException
     * @exception InvalidPathException
     */
    public Clause create( Proc proc, Partic partic, String tense, 
			  String adverb, String mood, String polarity, 
			  String complexity, boolean restrictiveness, 
			  Circum circum, PreMod preMod, String[] scope, 
			  FDRef ref) 
	throws InvalidConstructorException, InvalidPathException {
	return new ClauseH(proc, partic, tense, adverb, mood, polarity, 
			   complexity, restrictiveness, circum, 
			   preMod, scope, ref);
    }
}
