/*
 * @(#)ClauseH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * The clause category for a FD. This class is used whenever users
 * want to create a clause. As the minimum requirement, this class
 * must be used together with the Partic and Proc class. In addition
 * to setting the process and the participant of a clause, there are
 * methods in this class to set the various properties of a
 * clause. For methods that set the properties of a clause such as the
 * setTense(...) and setPolarity(...) methods, some of the possible
 * values/options for these methods are provided in the FDProperties
 * class.  This class implements the Clause interface and is
 * implemented in the form of a HashMap as it is inherrited from the
 * FDH class. This class consists of methods and constructors to set
 * the required components of a Clause. User can either use the
 * ClauseFactoryH class or directly use the constructor of this class
 * to create instances of ClauseH.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ClauseFactoryH
 * @see Proc
 * @see Partic
 * @see FDProperties
 */

public class ClauseH extends FDH implements Clause {
    /**
     * Default constructor which creates a default instance of
     * ClauseH.
     */
    public ClauseH() {
	super();
	this.put("cat", "clause");
    }

    /**
     * Constructor to create an instance of Clause class that fulfills
     * the minimum requirement of a clause, which consists of a
     * process and a participant. An InvalidConstructorException is
     * thrown if the proc and partic parameters are invalid. And an
     * InvalidPathException will be thrown when the program is failed
     * to use the putPathOverride() method in the FDH class to
     * construct the clause FD.
     *
     * @see FDH#putPathOverride
     * @param proc the process of the clause
     * @param partic the participant of the clause
     * @return an instance of Clause which consists of a given process
     * and a participant
     * @exception InvalidConstructorException
     */
    public ClauseH(Proc proc, Partic partic) 
	throws InvalidConstructorException {
	super();
	this.put("cat", "clause");
	if(proc == null || partic == null)
	    throw new InvalidConstructorException();

	this.setProc(proc);

	this.setPartic(partic);
    }

    /**
     * Constructor to create an instance of ClauseH class that gives
     * user the option to create an instance of ClauseH with all the
     * most common and frequently used properties. Among all the
     * parameters, only proc and partic are required. If users do not
     * want to specify certain properties, simply put a null as the
     * actual parameter. For some parameters such as the "tense",
     * "mood", the possible values/options can be found in the
     * FDProperties class.
     * @param proc the process of the clause
     * @param partic the participant of the clause
     * @param tense the tense of the clause
     * @param adverb the possible adverb for the clause
     * @param mood the possible mood of the clause
     * @param polarity the polarity of the clause
     * @param complexity set whether the clause is complex, true to be
     * complex and vice versa.
     * @param restrictiveness set whether the clause is restrictive,
     * true to be restrictive and vice versa.
     * @param circum the circumstance of the clause
     * @param preMod the predefined-modifier of the clause
     * @param scope specify the path to the hole and put them in the
     * right order into the array. For example, if the whole is
     * (partic (agent..))  then put String[] scope = {"partic",
     * "agent"}
     * @param ref the FD node which serves as the hole of the clause.
     * @return an instance of Clause which consists of all the user
     * specifications.
     * @exception InvalidConstructorException
     * @exception InvalidPathException
     */
    public ClauseH( Proc proc, Partic partic, String tense, String adverb,
		    String mood, String polarity, String complex, boolean
		    restrictiveness, Circum circum, PreMod preMod, String[]
		    scope, FDRef ref ) 
	throws InvalidConstructorException, InvalidPathException {
	super();
	try {
	    this.put("cat", "clause");
	    if(proc == null || partic == null)
		throw new InvalidConstructorException();

	    this.setProc(proc);

	    this.setPartic(partic);

	    if(tense != null && !tense.equals(""))
		this.setTense(tense);

	    if(adverb != null && !adverb.equals(""))
		this.setAdverb(adverb);

	    if(mood != null && !mood.equals(""))
		this.setMood(mood);

	    if(polarity != null && !polarity.equals(""))
		this.setPolarity(polarity);

	    if(complex != null && !complex.equals(""))
		this.setComplex(complex);

	    this.setRestrictive(restrictiveness);

	    if( circum != null )
		this.setCircum(circum);

	    if( preMod != null )
		this.setPreMod(preMod);

	    if( scope != null && (scope.length != 0) )
		this.setScope(scope);

	    if (ref != null)
		this.setScopeUsingRef(ref);
	}
	catch(Exception e) {
	}
    }

    /**
     * A method to set the process of a clause. (Required for a
     * clause)
     * @param proc the process to be specified
     */
    public void setProc(Proc proc) {
	this.put("process", proc);
    }

    /**
     * A method to set the participants of a clause. (Required)
     * @param partic the participants to be specified
     */
    public void setPartic(Partic partic) {
	this.put("partic", partic);
    }

    /**
     * A method to set the adverb of a clause. (Optional)
     * @param adverb the adverb to be inserted
     */
    public void setAdverb(String adverb) throws InvalidPathException {
	try {
	    this.putPathOverride("adverb lex", adverb);
	}
	catch (InvalidPathException e) {
	}
    }

    /**
     * A method to set the circumstance of a clause. (Optional)
     * @param circum the circumstance to be specified
     */
    public void setCircum(Circum circum) {
	this.put("circum", circum);
    }

    /**
     * A method to set the complexity of a clause. (Optional)
     * @param complex the complexity to be specified
     */
    public void setComplex(String complex) {
	this.put("complex", complex);
    }

    /**
     * A method to set the mood of a clause. (Optional)
     * @param mood the mood of the clause
     */
    public void setMood(String mood) {
	this.put("mood", mood);
    }

    /**
     * A method to set the polarity of a clause. By default, the
     * polarity is positive. (Optional)
     * @param polarity the polarity of a clause
     */
    public void setPolarity(String polarity) {
	this.put("polarity", polarity);
    }

    /**
     * A method to set the pre-modifier of a clause. (Optional)
     * @param preMod the pre-modifier to be specified
     */
    public void setPreMod(PreMod preMod) {
	this.put("pred-modif", preMod);
    }

    /**
     * A method to set the restrictiveness of a clause. (Optional)
     * @param restrictiveness true to be restrictive, false to be
     * non-restrictive
     */
    public void setRestrictive(boolean restrictive) {
	if(restrictive)
	    this.put("restrictive", FDProperties.RESTRICTIVE);
	else
	    this.put("restrictive", FDProperties.NON_RESTRICTIVE);
    }

    /**
     * A method to allow the user to set the tense of the clause. (Optional)
     * @param tense the tense of the clause
     */
    public void setTense(String tense) {
	this.put("tense", tense);
    }

    /**
     * A method to set the scope of a clause by specifying the path of the
     * holes in an array. (Optional)
     * @param scope specify the order of the roles by putting them in
     * the right order into the array. For example, (scope {^ A B})
     * will be {"A", "B"}.
     */
    public void setScope(String[] scope) {
	this.put("scope", scope);
    }

    /**
     * A method to set the scope of a clause by making a referrence to
     * the desired hole of the clause. If the hole is empty without
     * any properties, it will be a FDNull object. Otherwise, the hole
     * will just be a normal FD but it will be referrenced to the
     * clause as the hole.
     * @param ref the FD node which serves as the hole of the clause.
     * @see FDNull
     */
    public void setScopeUsingRef(FDRef ref) {
	this.put("scope", ref);
    }
}
