/*
 * @(#)FDH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;
import java.util.HashMap;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.Iterator;

/**
 * This is the class that implements the FD interface and is
 * inherited from the HashMap class.  This is the base class that all
 * kinds of FD will be inherrited from. It consists of methods to
 * modify and to get the key-value pairs of the map of the FD. The
 * toXML() method of this class has not be implemented due to time
 * constraint. All kinds of FD such as Clause, Common, Proc... has to
 * extend this class in order for the items to be able to hashed into
 * the hashmap.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see FD
 */

public class FDH extends HashMap implements FD {
    /**
     * Default constructor which creates a default instance of FDH.
     */
    public FDH() {
	super();
    }

    /**
     * A method to exports the FD in the form of XML. For now, the
     * implementation has not been supplied yet. An exception should
     * be thrown when the FD is invalid.
     * @return a XML object that represents the FD
     * @Exception InvalidPathException
     */
    public Object toXML() throws InvalidPathException {
	return null;
    }

    /**
     * A method for user to put a new value object into the key of a
     * FD that is specified by the given existing path. If the path is
     * not found, an exception will be thrown and no object will be
     * inserted. This put path method is implemented in the form of
     * recurrsion.  It recurrsively traversed to the given path and
     * put the value there.
     *
     * @param path A path name which users specify the path to the
     * desired node with a String array which each elelemnt in the
     * path array will be the name of the node to be traversed.
     * @param newValue A new value to be placed in the node of the
     * given path
     * @exception InvalidPathException
     */
    public void putPath(String[] path, Object newValue) 
	throws InvalidPathException {
	if( path == null || path.length == 0 )
	    throw new InvalidPathException();
	else if(path.length == 1 && this.containsKey(path[0]))
	    this.put(path[0], newValue);
	else if(this.containsKey(path[0]) && 
		(this.get(path[0]) instanceof FD)) {
	    String[] remainder = new String[path.length - 1];
	    System.arraycopy(path,1,remainder,0,remainder.length);
	    ((FDH)this.get(path[0])).putPath(remainder, newValue);
	} else
	    throw new InvalidPathException(path);
    }

    /**
     * A method for user to put a new value object into the key of a
     * FD that is specified by the given existing path. This method
     * basically use the stringToArray method to tokenized the given
     * path into an array and then use then pass it to the putPath
     * method.
     *
     * @see #putPath(String[] path, Object newValue)
     * @see #stringToArray(String path)
     * @param path A path name which users specify the path to the
     * desired node. The path is specified in one string with each
     * path separated by spaces.
     * @param newValue A new value to be placed in the node of the
     * given path
     * @exception InvalidPathException
     */
    public void putPath(String path, Object newValue) 
	throws InvalidPathException {
	this.putPath(this.stringToArray(path), newValue);
    }

    /**
     * A method for user to put a new value object associate with the
     * key of a FD that is specified by the given existing path. If
     * the given path is not found, it will be automatically created
     * and the new object will be inserted at the end of the
     * path. This is implemented in the form of recurrsion, it
     * recurrsively traversed to the given path and put the value
     * there. While it traverses, if the given path is not found in
     * the present FD, it will automatically create the abscent paths
     * along its way.
     *
     * @param path A path name which users specify the path to the
     * desired node with a String array which each elelemnt in the
     * path array will be the name of the node to be traversed.
     * @param newValue A new value to be placed in the node of the given path
     * @exception InvalidPathException
     */
    public void putPathOverride(String[] path, Object newValue) 
	throws InvalidPathException {
	if(path == null || path.length == 0)
	    throw new InvalidPathException();
	else if (path.length == 1)
	    this.put(path[0], newValue);
	else if(this.containsKey(path[0]) && 
		(this.get(path[0]) instanceof FD)) {
	    String[] remainder = new String[path.length - 1];
	    System.arraycopy(path,1,remainder,0,remainder.length);
	    ((FDH)this.get(path[0])).putPathOverride(remainder, newValue);
	} else {
	    FDH newFDH = new FDH();
	    this.put(path[0], newFDH);
	    String[] remainder = new String[path.length - 1];
	    System.arraycopy(path,1,remainder,0,remainder.length);
	    newFDH.putPathOverride(remainder, newValue);
	}
    }

    /**
     * A method for user to put a new value object associate with the
     * key of a FD that is specified by the given existing path. If
     * the given path is not found, it will be automatically created
     * and the new object will be inserted at the end of the
     * path. Same as that of putPath(), this modifies the string path
     * into the string[] equivalent and use the other putPathOverride
     * method to process the information.
     *
     * @param path A path name which users specify the path to the
     * desired node with a string which path names are separated by
     * spaces.
     * @param newValue A new value to be placed in the node of the
     * given path
     * @exception InvalidPathException
     */
    public void putPathOverride(String path, Object newValue) 
	throws InvalidPathException {
	this.putPathOverride(this.stringToArray(path), newValue);
    }

    /**
     * A method to verify whether the path exists in this FD. This
     * method recurssively looks for the given path within the map of
     * the given FD.
     *
     * @param path the path to be verified, specified by a string
     * array with the names of the nodes of the path.
     * @return a boolean of the verification result, true if the path
     * exists and vice versa.
     */
    public boolean checkPath(String[] path) {
	if(path == null || path.length == 0)
	    return false;

	if(path.length == 1)
	    return this.containsKey(path[0]);
	else if(this.containsKey(path[0])) {
	    if(this.get(path[0]) instanceof FD) {
		String[] newPath = new String[path.length-1];
		System.arraycopy(path,1,newPath,0,newPath.length);
		return ((FD)this.get(path[0])).checkPath(newPath);
	    } else
		return false;
	} else
	    return false;
    }

    /**
     * A method to verify whether the given path exists in this
     * FD. This method uses the stringToArray method to convert the
     * path string into a path array and then use the
     * checkPath(String[] array) method to compute the result.
     *
     * @see #stringToArray(String path)
     * @see #checkPath(String[] path)
     * @param path the path to be verified, specified by a string with
     * the names of the nodes of the path separated by spaces.
     * @return a boolean of the verification result
     */
    public boolean checkPath(String path) {
	return this.checkPath(this.stringToArray(path));
    }

    /**
     * A method to return the object that associates with the given
     * path. This method is implemented using recurssion; it
     * recurssively traverse the map within the FD using the given
     * path and obtain the object at the end of the path. An exception
     * is thrown when the given path is not be found.
     *
     * @param path the path of the object to be retrieved, specified
     * in a string array that stores each path node name.
     * @return value the object on that path.
     * @exception InvalidPathException
     */
    public Object getPath(String[] path) throws InvalidPathException {
	if(path == null || path.length == 0)
	    throw new InvalidPathException(path);

	if(path.length == 1)
	    return this.get(path[0]);
	else if(this.containsKey(path[0])) {
	    if(this.get(path[0]) instanceof FD) {
		String[] remainder = new String[path.length - 1];
		System.arraycopy(path,1,remainder,0,remainder.length);
		return ((FD)this.get(path[0])).getPath(remainder);
	    } else
		throw new InvalidPathException(path);
	} else
	    throw new InvalidPathException(path);
    }

    /**
     * A method to return the object that associates with the given
     * path. This method converts the input path to a string array and
     * uses the getPath(String[] path) to compute the result.
     *
     * @see #stringToArray(String path)
     * @see #getPath(String[] path)
     * @param path the path of the object to be retrieved, specified
     * in a string array that stores each path node name.
     * @return value the object on that path.
     * @exception InvalidPathException
     */
    public Object getPath(String path) throws InvalidPathException {
	return this.getPath(this.stringToArray(path));
    }

    /**
     * Override the toString() method inherrited from HashMap, this
     * method unifies the FD into its equivalent lisp-string
     * representation by calling the private toString method.
     * @see #toString(Map alreadyPrinted, Stack currentPath)
     * @return the lisp equivalent string representation of the FD
     */
    public String toString() {
	// the map that stores the map.entry that are already printed
	Map alreadyPrinted = new HashMap();
	// a stack that uses to store the current path
	Stack currentPath = new Stack();

	return toString(alreadyPrinted, currentPath);
    }

    /**
     * The private toString() method which constructs a lisp-string
     * representation of the FD. In addition to combining nodes and
     * putting them into the right paranthesized form, it keeps track
     * of the path that it currently traversed and the nodes that are
     * already printed so that we will not print re-used nodes and
     * also we are able to produce the correct statement for the holes
     * in a FD.
     * @param alreadyPrinted a HashMap that keeps track of the FD that
     * is already printed and processed
     * @param currentPath a stack which keeps track of the current
     * path that the method traverses
     * @return the lisp equivalent string representation of the FD
     */
    protected String toString(Map alreadyPrinted, Stack currentPath) {
	int max = this.size() - 1;
	StringBuffer buf = new StringBuffer();
	Iterator i = this.entrySet().iterator();

	// stores the map.Entry that are waiting to be printed
	List toBePrinted = new ArrayList();
	buf.append("(");
	for (int j = 0; j <= max; j++) {
	    Entry e = (Entry) (i.next());
	    Object key = e.getKey();
	    Object value = e.getValue();
	    currentPath.push(key);

	    if (alreadyPrinted.containsKey(value)) {
		//compute the path difference and print the difference
		//as the value.  the first parameter is the current
		//path and the second parameter is the path that is
		//already present. The result of this function is
		//always what is already in the printed path but what
		//is lack in the current path.
		buf.append("(" + key + " {" + 
			   computePathDifference(stackToPath(currentPath),
						 (String[])
						 alreadyPrinted.get(value))
			   + "})");
	    } else if( key.equals("lex") )
		buf.append("(" + key + " " + '"' + value + '"' + ")");
	    else if( (key.equals("describer") && (value instanceof String)) || 
		     (key.equals("attribute") && (value instanceof String)) ||
		     (key.equals("classifier") && (value instanceof String))  ||
		     (key.equals("prep") && (value instanceof String)) ||
		     (key.equals("time") && (value instanceof String)) ||
		     (key.equals("adverb") && (value instanceof String)) ) {
		buf.append("(" + key + " === " + '"' + value + '"' + ")");
	    } else if( key.equals("scope") && (value instanceof String[]) ) {
		// one way of setting the scope, i.e., the holes.

		String scopeString = "";
		String[] scope = (String[])value;
		for (int k = 0; k < scope.length; k++)
		    scopeString += " " + scope[k];
		buf.append("(" + key + " {^" + scopeString + "}" + ")");
	    } else if (value instanceof FDNull) {
		// another way of setting the holes and the scope.

		// silently ignore it, but record its position since
		// it is the hole
		alreadyPrinted.put(value,stackToPath(currentPath));
	    } else if (value instanceof FDRef) {
		FD ref = (FD)((FDRef)value).getRef();

		if(alreadyPrinted.containsKey(ref)) {
		    buf.append("(" + key + " {" + 
			       computePathDifference(stackToPath(currentPath), 
						     (String[])
						     alreadyPrinted.get(ref)) + 
			       "})");
		} else
		    toBePrinted.add(e);
	    } else if(value instanceof FD) {
		alreadyPrinted.put(value, stackToPath(currentPath));
		buf.append("(" + key + " " + 
			   ((FDH)value).toString(alreadyPrinted, currentPath) + 
			   ")");
	    } else
		buf.append("(" + key + " " + value + ")");
	    
	    currentPath.pop();

	    if (j < max)
		buf.append(" ");
	}

	Iterator it = toBePrinted.iterator();
	while(it.hasNext()) {
	    buf.append(" ");
	    Entry e = (Entry)it.next();
	    Object key = e.getKey();
	    Object value = e.getValue();
	    Object ref = ((FDRef)value).getRef();

	    currentPath.push(key);

	    buf.append("(" + key + " {" + 
		       computePathDifference(stackToPath(currentPath),
					     (String[])alreadyPrinted.get(ref)) +
		       "})");

	    currentPath.pop();
	}
	buf.append(")");

	return buf.toString();
    }

    /**
     * This method computes the difference between the current path
     * and the path that is already printed and returns their
     * difference. It is used to compute what needs to be put into the
     * scope statement and it is also used when constructing
     * reuse-able nodes. This method is used by the toString() method
     * to construct the lisp representation of the FD.
     * @see #toString(Map alreadyPrinted, Stack currentPath)
     * @param currentPath the current path that it has traversed to
     * @param alreadyPrintedPath the path that is already printed
     * @return a string consists of the difference between the two
     * paths with each path name separated by a space.
     */
    protected String computePathDifference(String[] currentPath, 
					   String[] alreadyPrintedPath) {
	int start = 0;
	String result = "^";

	while(currentPath[start].equals(alreadyPrintedPath[start]) && 
	      start < currentPath.length && 
	      start < alreadyPrintedPath.length)
	    start++;
	for(int a = start; a < alreadyPrintedPath.length; a++)
	    result += " " + alreadyPrintedPath[a];

	return result;
    }

    /**
     * A method that takes in a stack which stores the path names that
     * the method has traversed to and convert the path into a string
     * array. This method is used by the toString(Map alreadyPrinted,
     * Stack currentPath) method to construct the lisp representation
     * of the FD.
     * @see #toString(Map alreadyPrinted, Stack currentPath)
     * @param a stack which stores all the path names
     * @return the path that is represented as a string array.
     */
    protected String[] stackToPath(Stack stack) {
	String[] path = new String[stack.size()];
	System.arraycopy(stack.toArray(), 0, path, 0, path.length);
	return path;
    }

    /**
     * A method that takes in a path string and return a path string
     * array equivalent.
     * @param string the path string
     * @return the path string array
     */
    protected String[] stringToArray(String string) {
	StringTokenizer tokenizer = new StringTokenizer(string);
	int numTokens = tokenizer.countTokens();
	String[] pathArray = new String[numTokens];

	for( int i = 0; i < numTokens; i++ )
	    pathArray[i] = tokenizer.nextToken();

	return pathArray;
    }
}
