/*
 * @(#)ParticH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * The participant of a FD in a clause. This is used when user creates
 * a clause and thus is used together with the Proc and Clause
 * class. Depends on what the process is, users will only need to use
 * the ones that they need for that particular process.  However, here
 * i try to provide all kinds of information for users to composite
 * various kinds of participant. There are methods to set the subject
 * and object of the participant independently, and there are also
 * methods to set up particular subject/object pairs since for the
 * most cases, there usually has a pair of elements in a participant
 * of a clause. By using the "pair-methods" to setup participant also
 * help users to reduce the risk of grouping incompatible
 * subject/object pair. In the "pair-methods", i try to summarize in
 * what scenarios will user need to use that particular
 * method. However, the information that i gave there is not
 * complete. User should always follow the spec of SURGE since there
 * may be other different combinations. Moreover, in some cases, there
 * will be two pair of participants in each scenario. In those cases,
 * users can create a pair using the constructor and create another
 * pair using one of the set methods in this class.  This class is
 * implemented in the form of a HashMap which the values within a
 * Participant are hashed into the HashMaps. User can either use the
 * ParticFactoryH class or directly use the constructor of this class
 * to create instances of ParticH.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ParticFactoryH
 * @see Clause
 * @see Proc
 */

public class ParticH extends FDH implements Partic {
    /**
     * Constructor. It creates a default instance of ParticH.
     */
    public ParticH() {
	super();
    }

    /**
     * Constructor. It creates an instance of ParticH with the given
     * name and the body of the participant. Both parameters in the
     * constructors have to be present. This is used for clauses that
     * only have one participant.  If invalid name and body are
     * entered, an exception will be thrown.
     * @param particName the name of the type of participant
     * @param particbody the specification of the participant
     * @exception InvalidConstructorException
     * @return an instance of ParticH with the given participant's
     * name and body
     */
    public ParticH(String particName, FD particBody) 
	throws InvalidConstructorException {
	super();
	if(particName != null && !particName.equals("") && 
	   particBody != null)
	    this.put(particName, particBody);
	else
	    throw new InvalidConstructorException();
    }

    /**
     * A factory method which creates an instance of ParticH with the
     * given names and the bodies of the participant. There are a lot
     * of instances participants come in the form of pairs. This
     * factory method allows users to create participant pairs in one
     * line of code. All four parameters of the constructor have to be
     * present.  For clauses that have both participants, if invalid
     * names and body entries are entered, an exception will be
     * thrown.
     * @param subjectName the name of the type of the first participant
     * @param subjectBody the body specification of the first participant
     * @param objectName the name of the type of the other participant
     * @param objectBody the body specification of the other participant
     * @exception InvalidConstructorException
     * @return an instance of ParticH with the given partcipant pair.
     */
    public ParticH(String subjectName, FD subjectBody, String objectName,
		   FD objectBody) throws InvalidConstructorException {
	super();
	if ( objectName != null && !objectName.equals("") && 
	     objectBody != null && subjectName != null && 
	     !subjectName.equals("") && subjectBody != null ) {
	    this.put(subjectName, subjectBody);
	    this.put(objectName, objectBody);
	} else
	    throw new InvalidConstructorException();
    }

    /**
     * A method to set the agent of the participant
     * @param agent the agent of the participant
     */
    public void setAgent(FD agent) {
	this.put("agent", agent);
    }

    /**
     * A method to set the range of the participant
     * @param range the range of the participant
     */
    public void setRange(FD range) {
	this.put("range", range);
    }

    /**
     * A method to set the created of the participant
     * @param created the created of the participant
     */
    public void setCreated(FD created) {
	this.put("created", created);
    }

    /**
     * A method to set the affected of the participant
     * @param affected the affected of the participant
     */
    public void setAffected(FD affected) {
	this.put("affected", affected);
    }

    /**
     * A method to set the processor of the participant
     * @param processor the processor of the participant
     */
    public void setProcessor(FD processor) {
	this.put("processor", processor);
    }

    /**
     * A method to set the phenomenon of the participant
     * @param phenomenon the phenomenon of the participant
     */
    public void setPhenomenon(FD phenom) {
	this.put("phenomenon", phenom);
    }

    /**
     * A method to set the sayer of the participant
     * @param sayer the sayer of the participant
     */
    public void setSayer(FD sayer) {
	this.put("sayer", sayer);
    }

    /**
     * A method to set the addressee of the participant
     * @param addressee the addressee of the participant
     */
    public void setAddressee(FD addressee) {
	this.put("addressee", addressee);
    }

    /**
     * A method to set the verbalization of the participant
     * @param verbalization the verbalization of the participant
     */
    public void setVerbalization(FD verbalization) {
	this.put("verbalization", verbalization);
    }

    /**
     * A method to set the carrier of the participant
     * @param carrier the carrier of the participant
     */
    public void setCarrier(FD carrier) {
	this.put("carrier", carrier);
    }

    /**
     * A method to set the attribute of the participant
     * @param attribute the attribute of the participant
     */
    public void setAttribute(FD attribute) {
	this.put("attribute", attribute);
    }

    /**
     * A method to set the identified of the participant
     * @param identified the identified of the participant
     */
    public void setIdentified(FD identified) {
	this.put("identified", identified);
    }

    /**
     * A method to set the identifier of the partic
     * @param identifier the identifier of the partic
     */
    public void setIdentifier(FD identifier) {
	this.put("identifier", identifier);
    }

    /**
     * A method to set the possessor of the partic
     * @param possessor the possessor of the partic
     */
    public void setPossessor(FD possessor) {
	this.put("possessor", possessor);
    }

    /**
     * A method to set the possessed of the partic
     * @param possessed the possessed of the partic
     */
    public void setPossessed(FD possessed) {
	this.put("possessed", possessed);
    }

    /**
     * A method to set the located of the partic
     * @param located the located of the partic
     */
    public void setLocated(FD located) {
	this.put("located", located);
    }

    /**
     * A method to set the accompaniment
     * @param accompaniment the accompaniment
     */
    public void setAccompaniment(FD accompaniment) {
	this.put("accompaniment", accompaniment);
    }

    /**
     * A method to set the time
     * @param time the time
     */
    public void setTime(FD time) {
	this.put("time", time);
    }

    /**
     * A method to set the location
     * @param location the location
     */
    public void setLocation(FD location) {
	this.put("location", location);
    }

    /**
     * A method to set the agent-range pair; this method is used when
     * we have the effect-type of with-range.
     * @param agent the agent
     * @param range the range
     */
    public void setAgentRangePair(FD agent, FD range) {
	this.setAgent(agent);
	this.setRange(range);
    }

    /**
     * A method to set the agent-location pair; this method is used.
     * @param agent the agent
     * @param location the location
     */
    public void setAgentLocationPair(FD agent, FD location) {
	this.setAgent(agent);
	this.setLocation(location);
    }

    /**
     * A method to set the agent-created pair; this is used when we
     * have the creative effect-type.
     * @param agent the agent
     * @param created the created
     */
    public void setAgentCreatedPair(FD agent, FD created) {
	this.setAgent(agent);
	this.setCreated(created);
    }

    /**
     * A method to set the agent-affected pair; this is used when we
     * have the dispositive effect-type.
     * @param agent the agent
     * @param affected the affected
     */
    public void setAgentAffectedPair(FD agent, FD affected) {
	this.setAgent(agent);
	this.setAffected(affected);
    }

    /**
     * A method to set the processor-phenomenon pair; this is used
     * when we have the mental process type.
     * @param processor the processor
     * @param phenomenon the phenomenon
     */
    public void setProcessorPhenomenonPair(FD processor, FD phenom) {
	this.setProcessor(processor);
	this.setPhenomenon(phenom);
    }

    /**
     * A method to set the sayer-addressee pair; this is used when we
     * have the verbal process type.
     * @param sayer the sayer
     * @param addressee the addressee
     */
    public void setSayerAddresseePair(FD sayer, FD addressee) {
	this.setSayer(sayer);
	this.setAddressee(addressee);
    }

    /**
     * A method to set the sayer-verbalization pair; this is used when
     * we have the verbal process type.
     * @param sayer the sayer
     * @param verbalization the verbalization
     */
    public void setSayerVerbalizationPair(FD sayer, FD verbalization) {
	this.setSayer(sayer);
	this.setVerbalization(verbalization);
    }

    /**
     * A method to set the carrier-attribute pair; this is used when
     * we have the ascriptive process type and attributive
     * effect-type.
     * @param carrier the carrier
     * @param attribute the attribute
     */
    public void setCarrierAttributePair(FD carrier, FD attribute) {
	this.setCarrier(carrier);
	this.setAttribute(attribute);
    }

    /**
     * A method to set the identified-identifier pair; this is used
     * when we have the ascriptive process type and the equative
     * effect-type.
     * @param identified the identified
     * @param identifier the identifier
     */
    public void setIdentifiedIdentifierPair(FD identified, FD identifier) {
	this.setIdentified(identified);
	this.setIdentifier(identifier);
    }

    /**
     * A method to set the located-accompaniment pair; this is used
     * when you have the locative and accompaniment type.
     * @param located
     * @param accompaniment
     */
    public void setLocatedAccompanimentPair(FD located, FD accompaniment) {
	this.setLocated(located);
	this.setAccompaniment(accompaniment);
    }

    /**
     * A method to set the located-time pair; this is used when you
     * have a locative and temporal type.
     * @param located
     * @param time
     */
    public void setLocatedTimePair(FD located, FD time) {
	this.setLocated(located);
	this.setTime(time);
    }

    /**
     * A method to set the located-location pair; this is used when
     * you have a spatial and attributive type.
     * @param located
     * @param location
     */
    public void setLocatedLocationPair(FD located, FD location) {
	this.setLocated(located);
	this.setLocation(location);
    }

    /**
     * A method to set the possessor-possessed pair; this is used when
     * we have the possessive process type and attributive
     * effect-type.
     * @param possessor the possessor
     * @param possessed the possessed
     */
    public void setPossessorPossessedPair(FD possessor, FD possessed) {
	this.setPossessor(possessor);
	this.setPossessed(possessed);
    }
}
