/*
 * @(#)PersonalPronounH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * The personal-pronoun category. In general, a personal-pronoun is
 * used as participants of processes. For a personal-pronoun, at the
 * very least, user has to specify whether it is first, second or
 * third person and also whether it is plural. User can also set the
 * gender of the pronoun. The person and the gender of the pronoun can
 * be set using the static constants provided in this class. For
 * example, if user wants to set it to be first person, he/she can use
 * setPerson(PersonalPronoun.FIRST) to achieve the task. There are
 * methods to set the person, the plurality and the gender of the
 * personal-pronoun. A factory method is also provided to serve the
 * functionality of a constructor which user feeds in all the
 * properties of the desired pronoun as parameters to the method. User
 * can either use the PersonalPronounFactoryH class or directly use
 * the constructor of this class to create instances of
 * PersonalPronounH.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see Partic
 * @see PersonalPronounFactoryH
 */

public class PersonalPronounH extends FDH implements PersonalPronoun {
    /**
     * Constructor. It creates a default instance of PersonalPronounH.
     * @return a default instance of PersonalPronounH
     */
    public PersonalPronounH() {
	super();
	this.put("cat", "personal-pronoun");
    }

    /**
     * Constructor which computes and returns an instance of
     * PersonalPronounH with the given specification of the whether
     * the FD is in first, second or third person since specifying the
     * person is the least requirement in creating a PersonPronoun.
     * @param person specify whether the personal-pronoun should be
     * first, second or third person.  Can use the static constants as
     * parameters, example: PersonalPronoun.SECOND
     * @return an instance of PersonalPronounH with the given
     * parameters
     */
    public PersonalPronounH(String person) 
	throws InvalidConstructorException {
	super();
	this.put("cat", "personal-pronoun");
	if(person == null || person.equals(""))
	    throw new 
		InvalidConstructorException("Empty specification of the person.");
	else
	    this.setPerson(person);
    }

    /**
     * A factory method which computes and returns an instance of
     * PersonalPronounH.  Only person is strictly required to enter as
     * parameter because it is the minimum requirement to construct a
     * PersonPronoun object, the rest of the parameters are optional.
     * @param person specify whether the personal-pronoun should be
     * first, second or third person.  Can use the static constants as
     * parameters, example: PersonalPronoun.SECOND
     * @param plurality specify whether the personal-pronoun should be
     * plural or singular.
     * @param gender specify the gender of the personal-pronoun, can
     * use the static constants in this class (example:
     * PersonalPronoun.NEUTRAL) to feed in as parameters
     * @return an instance of PersonalPronounH with the given
     * parameters
     */
    public PersonalPronounH(String person, boolean plurality, String
			    gender) throws InvalidConstructorException {
	super();
	this.put("cat", "personal-pronoun");
	if(person == null || person.equals(""))
	    throw new 
		InvalidConstructorException("Empty specification of the person.");
	this.setPerson(person);

	this.setPlural(plurality);

	if(gender != null && !gender.equals(""))
	    this.setGender(gender);
    }

    /**
     * A method to set the person of the personal-pronoun.
     * @param person set whether the personal-pronoun to be first,
     * second or third person. User can use the static constants in
     * this class as parameters.
     */
    public void setPerson(String person) {
	this.put("person", person);
    }

    /**
     * A method to set whether the personal-pronoun is plural. User
     * can use the constants in the FDProperties to set the pluraity.
     * @param pluraity true to be plural and false to be singular
     * @see FDProperties
     */
    public void setPlural(boolean plural) {
	if(plural)
	    this.put("number", FDProperties.PLURAL);
	else
	    this.put("number", FDProperties.SINGULAR);
    }

    /**
     * A method to set the gender of the personal-pronoun.
     * @param gender can use the static variables in this class as
     * paratmers
     */
    public void setGender(String gender) {
	this.put("gender", gender);
    }
}
