/*
 * @(#)ProcH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.hashImpl;
import edu.columbia.surge.*;

/**
 * The process of a FD. This is used together with the Clause and the
 * Partic class in order to create a clause sentence. There are
 * methods in this class to set and to create various properties of a
 * process. Moreover, this interface also provides various constants
 * to help users setting the properties of a process. For example,
 * when user wants to set the process type of the Process to be
 * material, user can just use setProcessType(Proc.MATERIAL). Factory
 * methods are also provided to help users to construct instance of
 * Proc in a more efficient fashion.  User can either use the ProcH
 * class or directly use the constructor of this class to create
 * instances of ProcH.
 *
 * @author Howard Chu (hhc42@columbia.edu)
 * @version alpha 0.1
 *
 * @see ProcFactoryH
 * @see Clause
 * @see Partic
 * @see #setProcessType(String processType)
 * @see #MATERIAL
 */

public class ProcH extends FDH implements Proc {
    /**
     * Constructor which creates a default instance of ProcFactoryH.
     * @return a default instance of ProcFactoryH
     */
    public ProcH() {
	super();
    }

    /**
     * Constructor which takes in the lexical entry and the process
     * type as parameters to compute and return an instance of
     * ProcH. An exception will be thrown if the parameters are
     * invalid.
     * @param type the process type
     * @param lex the lexical entry/verb of the process
     * @exception InvalidConstructorException
     * @return an instance of Proc with the given process type and the
     * lexical entry
     */
    public ProcH(String type, String lex) 
	throws InvalidConstructorException {
	super();

	if(type == null || type.equals("") || lex == null || lex.equals(""))
	    throw new InvalidConstructorException();

	this.setProcessType(type);

	this.setLex(lex);
    }

    /**
     * Constructo which computes and returns an instance of ProcH that
     * gives user the option to create an instance of ProcH with all
     * the most common and most frequently used properties. If user
     * does not want to specify certain properties, simply put in a
     * null for that entry. However, as the minimum requirement for
     * constructing a proc, user has to at least supply the type
     * (process type) and the lexical entry. Otherwise, an
     * InvalidConstructorException will be thrown.
     * @param type the process type
     * @param lex the lexical entry/verb of the process
     * @param effect_type the effect type of the process
     * @param agentive set whether the process is agentive
     * @param effective set whether the process is effective
     * @param mode the mode of the process
     * @param relation_type the relation_type of the process
     * @exception InvalidConstructorException
     * @return an instance of Proc with all the user-defined
     * properties.
     */
    public ProcH(String type, String lex, String effect_type, boolean
		 agentive, boolean effective, String mode, String
		 relation_type, String polarity) 
	throws InvalidConstructorException {
	super();

	if(type == null || type.equals("") || lex == null || lex.equals(""))
	    throw new InvalidConstructorException("No type.");

	this.setProcessType(type);

	this.setLex(lex);

	if(effect_type != null && !effect_type.equals(""))
	    this.setEffectType(effect_type);

	this.setAgentive(agentive);

	this.setEffective(effective);

	if(mode != null && !mode.equals(""))
	    this.setMode(mode);

	if(relation_type != null && !relation_type.equals(""))
	    this.setRelationType(relation_type);

	if(polarity != null && !polarity.equals(""))
	    this.setPolarity(polarity);
    }

    /**
     * A method to set the type of the process
     * @param type the type of the process
     */
    public void setProcessType( String processType ) {
	this.put("type", processType);
    }

    /**
     * A method to set the polarity of the process
     * @param polarity the options are given as static variables in
     * FDProperties interface.
     * @see FDProperties
     */
    public void setPolarity(String polarity) {
	this.put("polarity", polarity);
    }

    /**
     * A method to set the effect-type of the process
     * @param effectType the effect-type of the process
     */
    public void setEffectType(String effectType) {
	this.put("effect-type", effectType);
    }

    /**
     * A method to set the lexical of the process
     * @param lex the lexical word of the process
     */
    public void setLex(String lex) {
	this.put("lex", lex);
    }

    /**
     * A method to set the mode of the process
     * @param mode the mode of the process
     */
    public void setMode(String mode) {
	this.put("mode", mode);
    }

    /**
     * A method to set whether the process is going to be effective
     * @param effective true to set to effective, and vice versa.
     */
    public void setEffective(boolean effective) {
	if(effective)
	    this.put("effective", ProcH.EFFECTIVE);
	else
	    this.put("effective", ProcH.NON_EFFECTIVE);
    }

    /**
     * A method to set whether the process is going to be agentive
     * @param agentive true to set to agentive, and vice versa
     */
    public void setAgentive(boolean agentive) {
	if(agentive)
	    this.put("agentive", ProcH.AGENTIVE);
	else
	    this.put("agentive", ProcH.NON_AGENTIVE);
    }

    /**
     * A method to set the relation-type of the process
     * @param relation_type
     */
    public void setRelationType(String relation_type) {
	this.put("relation-type", relation_type);
    }
}
