/*
 * @(#)WeakHashFDH.java alpha0.1
 *
 * Copyright 2002 Columbia University, All Rights Reserved.
 *
 * This software is distributed under the LGPL.
 *
 */

package edu.columbia.surge.unifier;
import edu.columbia.surge.*;
import java.util.HashMap;
import java.util.StringTokenizer;
import java.util.Iterator;
import java.util.Stack;
import java.util.Map;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;
import java.util.Collection;


/**
 * An FD with a weak (fast) hash and structural equals.
 *
 * @author Pablo Ariel Duboue (pablo@cs.columbia.edu)
 * @version alpha 0.1
 */

class WeakHashFDH implements FD {
    /** The wrapped FD. */
    protected FD fd;
    /** The cached hash value. */
    protected int hashCode;

    /**
     * Build a Weak Hash FD from an existing FD. The FD is kept in an
     * variable and a hash value is computed using the computeHash
     * method.
     *
     * @param fd a functional description, in any implementation.
     * @see #computeHash
     */
    public WeakHashFDH(FD fd) {
	this.fd = fd;
	hashCode = computeHash(fd);
    }

    /**
     * Not implemented.
     */
    public Object toXML() throws InvalidPathException {
	return null;
    }

    /**
     * Using the original FD methods.
     */
     public void putPath(String[] path, Object newValue) 
	throws InvalidPathException {
	fd.putPath(path,newValue);
    }
    /**
     * Using the original FD methods.
     */
    public void putPath(String path, Object newValue) 
	throws InvalidPathException {
	fd.putPath(path,newValue);
    }
    /**
     * Using the original FD methods.
     */
    public void putPathOverride(String[] path, Object newValue) 
	throws InvalidPathException {
	fd.putPathOverride(path,newValue);
    }
    /**
     * Using the original FD methods.
     */
    public void putPathOverride(String path, Object newValue) 
	throws InvalidPathException {
	fd.putPathOverride(path,newValue);
    }
    /**
     * Using the original FD methods.
     */
    public boolean checkPath(String[] path) {
	return fd.checkPath(path);
    }
    /**
     * Using the original FD methods.
     */
    public boolean checkPath(String path) {	
	return fd.checkPath(path);
    }
    /**
     * Using the original FD methods.
     */
    public Object getPath(String[] path) throws InvalidPathException {
	return fd.getPath(path);
    }
    /**
     * Using the original FD methods.
     */
    public Object getPath(String path) throws InvalidPathException {
	return fd.getPath(path);
    }
    /**
     * Using the original FD methods.
     */
    public String toString() {
	return fd.toString();
    }

    /**
     * Using the original FD methods.
     */
    public void clear() { fd.clear(); }
    /**
     * Using the original FD methods.
     */
    public boolean containsKey(Object key) { return fd.containsKey(key); }
    /**
     * Using the original FD methods.
     */
    public boolean containsValue(Object value) { return fd.containsValue(value); }
    /**
     * Using the original FD methods.
     */
    public Set entrySet() { return fd.entrySet(); }
    /**
     * Using the original FD methods.
     */
    public Object get(Object key) { return fd.get(key); }
    /**
     * Returns the cached hash code computed by computeHash.
     *
     * @see #computeHash
     */
    public int hashCode() { return hashCode; }
    /**
     * Using the original FD methods.
     */
    public boolean isEmpty() { return fd.isEmpty(); }
    /**
     * Using the original FD methods.
     */
    public Set keySet() { return fd.keySet(); }
    /**
     * Using the original FD methods.
     */
    public Object put(Object key, Object value) { return fd.put(key,value); }
    /**
     * Using the original FD methods.
     */
    public void putAll(Map t) { fd.putAll(t); }
    /**
     * Using the original FD methods.
     */
    public Object remove(Object key) { return fd.remove(key); }
    /**
     * Using the original FD methods.
     */
    public int size() { return fd.size(); }
    /**
     * Using the original FD methods.
     */
    public Collection values() { return fd.values(); }

    /**
     * One of the class functionalities is implemented here. This
     * method will invoke the checkEquals method to do a set-based
     * check for equality.
     *
     * @param o an java.lang.Object in order to comply with the
     * definition of the equals method. Must be an FD.
     */
    public boolean equals(Object o) {
	return checkEquals(fd,o);
    }

    /**
     * The protected method that checks for equality, invoked
     * recursively.
     */
    protected boolean checkEquals(Object o1, Object o2) {
	if ((o1 instanceof FD) && (o2 instanceof FD)) {
	    FD fd1 = (FD)o1;
	    FD fd2 = (FD)o2;

	    Set entries = fd1.entrySet();
	    if (entries.size() != fd2.entrySet().size())
		return false;

	    Iterator it = entries.iterator();
	    while (it.hasNext()) {
		Map.Entry e = (Map.Entry)it.next();
		if (!(fd2.containsKey(e.getKey())))
		    return false;
		if (!checkEquals(e.getValue(),fd2.get(e.getKey())))
		    return false;
	    }
	    return true;
	} else
	    return o1.equals(o2);
    }

    /**
     * The recursive method used to compute a hash.  It computes the
     * sum of the product of the hash of the keys and the computeHash
     * of the values in the FD.
     * @param o an object to compute the hash. It will return it
     * hashCode if it is not an FD.
     */
    protected int computeHash(Object o) {
	if (o instanceof FD) {
	    int result = 0;
	    Set entries = ((FD)o).entrySet();
	
	    Iterator it = entries.iterator();
	    while (it.hasNext()) {
		Map.Entry e = (Map.Entry)it.next();
		result += e.getKey().hashCode() * computeHash(e.getValue());
	    }
	    return result;
	} else
	    return o.hashCode();
    }
}
